%******************************************************************************;
%* SAS macro DisTrans vs. 0.0.1 , 27 Jun 2011, by Jim Groeneveld, Netherlands *;
%******************************************************************************;
%*                                                                            *;
   %MACRO   DisTrans   /* Normality and transformation analysis               */
/* -------------------------------------------------------------------------- */
/* ---------- A l w a y s   s p e c i f i e d   a r g u m e n t s ----------- */
/* Argument Default       Description and remarks      Required/Optional: R/O */
/* -------- --------      --------------------------------------------------- */
(  Data    = _LAST_    /* input data set with variables to analyse          R */
,  Stats   =           /* output dataset name with summarised statistics    R */
,  Subject =           /* Subject ID variable (e.g. SubjectNumberStr)       R */
,  Variable=           /* Variable name of variable to analyse              R */
/*                                                                            */
/* ------ F r e q u e n t l y   s p e c i f i e d   a r g u m e n t s ------- */
/* Argument Default       Description and remarks      Required/Optional: R/O */
/* -------- --------      --------------------------------------------------- */
/*                                                                            */
/* ---- O c c a s i o n a l l y   s p e c i f i e d   a r g u m e n t s ----- */
/* Argument Default       Description and remarks      Required/Optional: R/O */
/* -------- --------      --------------------------------------------------- */
,  MaxWidth = 5        /* maximum width for use to label points in plots    O */
,  Orientation =       /* Horizontal, if empty then Vertical Boxplots         */
,  Delete  = No        /* Yes / No: deletion of Stats dataset when starting O */
/*                                                                            */
/* ---------- R a r e l y   s p e c i f i e d   a r g u m e n t s ----------- */
/* Argument Default       Description and remarks      Required/Optional: R/O */
/* -------- --------      --------------------------------------------------- */
,  Version = 0.0.1     /* forced version specification for version control  O */
/*                                                                            */
/* ----------- N e v e r   s p e c i f i e d   a r g u m e n t s ------------ */
/* Argument Default       Description and remarks      Required/Optional: R/O */
/* -------- --------      --------------------------------------------------- */
,  Lambda  = .N        /* basic value, DON'T CHANGE, for recursive call only  */
,  Trnsfrmd= .O        /* basic value, DON'T CHANGE, for recursive call only  */
,  Constant= /*empty*/ /* basic value, DON'T CHANGE, for recursive call only  */
,  ForMyUse=           /* argument for the program's use only, do not use it  */
,  Debug   = 0         /* Debug level; for debugging purposes only            */
)/ DES     =              'Normality and transformation analysis' /*Store*/    ;
%*                                                                            *;
%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*;
%*                                                                            *;
%* File       : DisTrans.SAS / DisTrans.001                                   *;
%* Directory  : .................                                             *;
%*                                                                            *;
%* Purpose    : Normality and transformation analysis.                        *;
%*                                                                            *;
%* Programmer : Jim Groeneveld                                                *;
%* Date       : 27 Jun 2011                                                   *;
%* Version    : 0.0.1                                                         *;
%*                                                                            *;
%* SAS version: 6.12 under Windows 95 / 8.2 under Windows 2K / XP             *;
%* SAS version: 9.1/9.2 under MS Windows XP, Professional Edition / EG 4.3/XP *;
%*                                                                            *;
%* Validation : -                                                             *;
%*                                                                            *;
%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*;
%*                                                                            *;
%* Output:                                                                    *;
%* =======                                                                    *;
%*                                                                            *;
%* Examples:                                                                  *;
%* =========                                                                  *;
%*                                                                            *;
%* Minimum specification:                                 Macro call:         *;
%* ----------------------                                 -----------         *;
%*                                                                            *;
%* More extensive specification:                          Macro call:         *;
%* -----------------------------                          -----------         *;
%*                                                                            *;
%* Remarks:                                                                   *;
%* ========                                                                   *;
%*                                                                            *;
%* Used auxiliary macros:                                                     *;
%* ----------------------                                                     *;
%* = internal: -                                                              *;
%* = attached: -                                                              *;
%* = external: - UstrMcmp                                                     *;
%*                                                                            *;
%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*;
%*                                                                            *;
%* History of DisTrans:                                                       *;
%* ====================                                                       *;
%* version  date         description of changes, additions, improvements      *;
%* -------  -----------  ---------------------------------------------------- *;
%* 0.0.0    29 Apr 2011  first working release                                *;
%* 0.0.1    27 Jun 2011  added (this) macro header and usual arguments        *;
%*                       added Delete argument to delete Stats ds at start,   *;
%*                       added prevention of indefinite loop by any cause     *;
%*                                                                            *;
%* Known bugs:                                                                *;
%* ===========                                                                *;
%*                                                                            *;
%* Future features:                                                           *;
%* ================                                                           *;
%* - check specification of required argument values (VERY, VERY FIRST T DO)  *;
%* - check existence of datasets and variables (VERY FIRST THING TO DO)       *;
%* - input of variable list instead of a single variable                      *;
%* - Overwrit argument to prevent accidental deletion and append to Stats     *;
%* - more flexible titles with TITLES argument                                *;
%* - more documentation in header                                             *;
%* - add more descriptive comment                                             *;
%* - (fix possible bugs)                                                      *;
%* - (add possible features)                                                  *;
%*                                                                            *;
%******************************************************************************;

%* OPTIONS MPRINT MERROR SERROR MLOGIC SYMBOLGEN MACROGEN;

%*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~;
%* LOCAL macro variables declarations;
%*___________________________________;

  %LOCAL   /* Define internal macro variables explicitely local */
  MacName  /* This macros external name: DisTrans */
  MacVs    /* Version number major.minor of macro: 0.0.1 */
  MacDate  /* Date of this version of macro: 27 Jun 2011 */
  Err      /* text 'E R R O R' as consecutive characters */
  Warn     /* text 'W A R N I N G' as consecutive characters */
  ;
  %LET MacName  = DisTrans;
  %LET MacVs    = 0.0.1;
  %LET MacDate  = 27 Jun 2011;
  %LET Err      = %SYSFUNC(COMPRESS(E R R O R, %STR( )));
  %LET Warn     = %SYSFUNC(COMPRESS(W A R N I N G, %STR( )));

  %IF ( %UPCASE(&ForMyUse) = COMPILE ) %THEN %DO;
    %*Spacing0 (ForMyUse=Compile);
    %*First (ForMyUse=Compile);
    %GOTO Finish;
  %END;
 
  %IF (&Debug EQ 1 OR &Debug GT 11) %THEN %DO;
    OPTIONS MPRINT MERROR SERROR MLOGIC SYMBOLGEN MACROGEN;
  %END;

%*~~~~~~~~~~~~~~~~~~~~~~~~~;
%* Macro header in LOG file;
%*_________________________;

  %PUT;
  %PUT |~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~|;
  %PUT |   Macro &MacName, vs. &MacVs, by Jim Groeneveld, &MacDate  |;
  %PUT |             Normality and transformation analysis            |;
  %PUT |______________________________________________________________|;
  %PUT;

%* Feedback of specified arguments;
  %PUT &MacName: Review of argument specifications (incl. defaults);
  %PUT &MacName: Data     = &Data    ;
  %PUT &MacName: Stats    = &Stats   ;
  %PUT &MacName: Subject  = &Subject ;
  %PUT &MacName: Variable = &Variable;
  %PUT &MacName: MaxWidth = &MaxWidth;
  %PUT &MacName: Orientation=&Orientation;
  %PUT &MacName: Delete   = &Delete  ;
  %PUT &MacName: Debug    = &Debug   ;
  %PUT &MacName: Version  = &Version ;
  %PUT; 
  %PUT &MacName: See SAS log for other characteristics;

%HeadSkip:

  %IF (%LENGTH(&Version) AND &Version NE &MacVs) %THEN %DO;
    %PUT *** &MacName *** &ERR: Specified version &Version does not match &MacVs,;
    %PUT .                       macro &MacName will abort;
    %GOTO Finish;
  %END;

  %LOCAL   /* Define internal macro variables explicitely local */
    NofCases
    TrnsType
    TrnsText
    Best_Fmt
    Dataset
    UniStat
    Selected
    _Err_
    ;
  %LET _Err_    = %SYSFUNC(COMPRESS(E R R O R =, %STR( )));
  %LET Dataset  = __Data__;
  %LET UniStat  = _Single_;
  %LET Selected = _Chosen_;

  %IF (&Debug) %THEN %DO;
    LIBNAME Current '.'; * ---debug---;
  %END;

%* Process keyword _LAST_ with Data;
  %IF (%UPCASE(&Data) EQ _LAST_) %THEN %LET Data = &SYSLAST;

%*------;
%* Core ;
%*------;

  %IF (%UstrMcmp(&Delete,Yes)) %THEN %_Delete_ (&Stats);

%* - - - - - - - - ;
%* D i s T r a n s ; %* --------- MACRO DisTrans --------- ;
%* - - - - - - - - ;

  %PUT =========> Macro DisTrans / start <=========;
  %PUT Data=&Data Variable=&Variable Trnsfrmd=&Trnsfrmd
    Constant=&Constant;
  %IF (%SYSEVALF(&Trnsfrmd EQ .O)) %THEN %DO;
    %LET TrnsType = 0; %* yet untransformed (1st round);
    %LET TrnsText = original;
  %END;
  %ELSE %IF (%SYSFUNC(VERIFY(&Trnsfrmd, -.0123456789))) %THEN %DO;
    %LET TrnsType = 2;
    %LET TrnsText = &Trnsfrmd transformed;
  %END;
  %ELSE %DO;
    %LET TrnsType = 1;
    %LET TrnsText = Box Cox Lambda=&TrnsFrmd transformed;
  %END;

  %PUT ==> Macro DisTrans, Data=&Data, Variable=&Variable, Lambda=&Lambda,;
  %PUT ==> Trnsfrmd=&Trnsfrmd, Constant=&Constant, TrnsType=&TrnsType;

* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - ;
* Initially the analysis is run on original, untransformed variables  ;
* indicated by macro argument Trnsfrmd=.O (original).                 ;
* Next, most of the analysis is done on transformed variables,        ;
* indicated by a non-empty, specified macro argument Trnsfrmd.        ;
* Trnsfrmd is expected to have the following values:                  ;
* - any numeric value, representing Lambda from Box Cox analysis      ;
* - any of the words LOG, EXP ....................................... ;
*   representing the indicated transformation to work with.           ;
* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - ;

* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -;
* Firstly verify whether outcome parameter of a continuous nature [defined as] ;
* - counts with the mean value above 5 ?????????????????? CHECK!               ;
* - ordinal parameters of 7 or more levels - CHECK!                            ;
* (For these a continuous nature is assumed that still has to be verified.)    ;
* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -;

* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -;
* Verify frequency distribution by visual inspection of:                       ;
* - boxplots                                                                   ;
* - histograms                                                                 ;
* - Q-Q plots                                                                  ;
* - skewness and curtosis                                                      ;
* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -;

* Obviously non-normal distributions may be attempted to be normal transformed.;

* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - ;
* Firstly create constant dummy (group) variable for box plots  ;
* and (optionally) transform original variable if applicable.   ;
* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - ;

* Statistical analyses per (numerical) variable programmed here;

PROC FORMAT;
  VALUE _Lambda
   .N = '(see below)'
   OTHER = [F4.1]
  ;
  VALUE _Trnsfmd
   .O = 'original'
    1 = '1, no transf.'
    2 = '2, square'
   .5 = '.5, sq. root'
    0 = '0, nat. log'
  -.5 = '-.5, inv. sqrt'
   -1 = '-1, inverse'
   -2 = '-2, inv. sq.'
  OTHER = '(see below)'
  ;
RUN;

%PUT =========> Data step Dataset (1) Variable=&Variable <=========;
DATA &Dataset;
  SET &Data (KEEP=&Variable &Subject);
  Variable = "&Variable";
* Transform original variable if applicable;
  %IF (&TrnsType EQ 1) %THEN %DO; %* Box Cox transformation;
    %IF (%SYSEVALF(&Trnsfrmd EQ 0)) %THEN %DO; /* (decimal values) */
      &Variable = LOG(&Variable + &Constant);
    %END;
    %ELSE %DO;
      &Variable = ((&Variable + &Constant)**&Trnsfrmd - 1) / &Trnsfrmd;
    %END;
  %END;
  %ELSE %IF (&TrnsType EQ 2) %THEN %DO; %* explicitly defined transformations;
    * not done;
  %END;
  LENGTH IDvariable $16;
  IDvariable = TRIM(LEFT(&Subject)) || '~' || LEFT(PUT(&Variable,BEST12.));
RUN;

%PUT =========> Process empty dataset &Data (due to Where) <=========;
%N0Cases (Data=&Dataset, Returned=NofCases,
          Message="Data=&Data empty, (no baseline records?)~no analysis on Variable=&Variable");
%IF (&NofCases LE 0) %THEN %GOTO Skipped;

* Skewness and kurtosis / histogram / Q-Q plot;

TITLE2 "Distribution of &TrnsText &Variable from &Data";

%PUT =========> PROC UNIVARIATE Dataset Variable=&Variable <=========;
PROC UNIVARIATE DATA=&Dataset /*(OBS=10)*/ NOPRINT NORMALTEST;
  VAR &Variable; * { (DROP=_TYPE_) applied to PROC MEANS } ;
  OUTPUT OUT=&UniStat /*(WHERE=(StdDev))*/ N=ValidNum NMISS=Missings
         MEAN=Mean STD/*DEV*/=StdDev MIN=Minimum MAX=Maximum MEDIAN=Median
         KURTOSIS=Kurtosis SKEWNESS=Skewness NORMALTEST=Normal PROBN=ProbNorm;
  HISTOGRAM / NORMAL (COLOR=green) CTEXT=green;
  QQPLOT / NORMAL (COLOR=red) CTEXT=red;
  INSET N='Count' MEAN='Mean' STD='StdDev' SKEWNESS='Skewness' KURTOSIS='Kurtosis';
  INSET NORMALTEST='NormalTest' PNORMAL='Pnormal' PROBN='ProbN';
RUN;

* - - - - - - - - - - - - - - - ;
* A g g r e g a t e d   d a t a ;
* - - - - - - - - - - - - - - - ;

* Find format information from PROC CONTENTS;
PROC CONTENTS DATA=&Dataset
               OUT=&Selected
    (KEEP = LibName MemName Name Label Type Format FormatL FormatD Length)
              NOPRINT;
RUN;

%PUT =========> Data step Statistics + Selected Variable=&Variable <=========;
* Add variables Dataset and Variable to &UniStat;
DATA &UniStat (DROP=LibName MemName Name); * one record dataset, one variable ;
  LENGTH Dataset Variable $ 32;
  Dataset  = "&Data";
  Variable = "&Variable";
  Lambda   = &Lambda;
  FORMAT Lambda _Lambda.;
  Trnsfrmd = &Trnsfrmd;
  FORMAT Trnsfrmd _Trnsfmd.;
  SET &UniStat;
  SET &Selected (WHERE=(UPCASE(MemName) EQ UPCASE("&Dataset") /* work dataset */
                    AND UPCASE(Name) EQ UPCASE("&Variable"))); * add fmt info;

* [HoV test(s), Cochran, Hartley, Bartlett?];

* Divide Skewness by its SES (a result GT 2 signifies non-normality);
* - - - - - - - - - - - - - - - - - - - - - - - - - ;
* http://jalt.org/test/bro_1.htm                    ;
* http://www.tc3.edu/instruct/sbrown/stat/shape.htm ;
* - - - - - - - - - - - - - - - - - - - - - - - - - ;
* SES: SQRT ( 6n(n-1) / ((n-2)(n+1)(n+3)) ), about SQRT ( 6/n );
  SES = SQRT ( 6*ValidNum*(ValidNum-1) / ((ValidNum-2)*(ValidNum+1)*(ValidNum+3)) );
  zSkewSES = Skewness / SES;

* Divide Kurtosis by its SEK (a result GT 2 signifies non-normality);
* - - - - - - - - - - - - - - - - - - - - - - - - - ;
* http://jalt.org/test/bro_1.htm                    ;
* http://www.tc3.edu/instruct/sbrown/stat/shape.htm ;
* - - - - - - - - - - - - - - - - - - - - - - - - - ;
* SEK: 2 x SES x SQRT ( (n-1) / ((n-3)(n+5)) ), about SQRT ( 24/n );
  SEK = 2 * SES * SQRT ( (ValidNum*ValidNum-1) / ((ValidNum-3)*(ValidNum+5)) );
  zKurtSEK = Kurtosis / SEK;

  LENGTH Advice $ 13;
  %IF (&TrnsType EQ 1) %THEN %DO; %* Box Cox transformation;
    IF (&Lambda LT -3 OR &Lambda GT 3) THEN Advice="Nonparametric";
    ELSE IF (zSkewSES GE -2 AND zSkewSES LE 2 AND
             zKurtSEK GE -2 AND zKurtSEK LE 2) THEN Advice = "Parametric OK";
    ELSE Advice = "Parametric???";
  %END;
  %ELSE %IF (&TrnsType EQ 0) %THEN %DO;
    Advice = '(see below)';
  %END;

  LABEL Dataset  = 'Dataset name'
        Variable = 'Variable name'
        Lambda   = 'Box Cox Lambda'
        Trnsfrmd = 'Categorised Lambda' /* -2 -1 -.5 0 .5 1 2 */
        ValidNum = 'Number of Valid observations'
        Missings = 'Number of Missing observations'
        Mean     = 'Mean value'
        StdDev   = 'Standard deviation'
        Minimum  = 'Minimum value'
        Maximum  = 'Maximum value'
        Median   = 'Median value'
        Kurtosis = 'Kurtosis'
        Skewness = 'Skewness'
        Normal   = 'Normality test statistic'
        ProbNorm = 'Deviation from Normality p-value'
        SES      = "Standard &_Err_ of Skewness"
        SEK      = "Standard &_Err_ of Kurtosis"
        zSkewSES = 'test statistic Skewness/SES'
        zKurtSEK = 'test statistic Kurtosis/SEK'
        Advice   = 'Advice for statistical analysis'
  ;
  FORMAT ProbNorm Pvalue6.4;

* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -;
* Calculate optimal format for variable based on maximum value   ;
* for use with labeling points in plots.                         ;
* If a format exists (already associated) then use that instead. ;
* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -;
  LENGTH Best_Fmt $8.;
  Best_Fmt = TRIM(LEFT(Format)) || TRIM(LEFT(PUT(FormatL,8.))) || '.' ||
                                   TRIM(LEFT(PUT(FormatD,8.)));
  IF (Type EQ 1 AND (NOT FormatL)) THEN DO; * (both if FormatL is 0 or missing);
    IntWidth = MAX(0,FLOOR(LOG10(ABS(Maximum+(NOT Maximum)))))+1+(Maximum LT 0);
    FmtWidth = MAX(IntWidth, &MaxWidth);
    DecWidth = MAX(0, &MaxWidth-IntWidth-1);
    Best_Fmt = TRIM(LEFT(Format)) ||
               TRIM(LEFT(PUT(FmtWidth,8.))) || '.' ||
               TRIM(LEFT(PUT(DecWidth,8.)));
    CALL SYMPUT ('Best_Fmt',Best_Fmt);
  END; * no provision yet for character (categorial) data formats;
* DROP Best_Fmt FmtWidth IntWidth DecWidth; * $$$ activate $$$;

RUN; * data step recreating 'Statistics' per variable, one record;

* Box plot;

TITLE2 "Box plot of &TrnsText &Variable from &Data";

ODS GRAPHICS ON; * to enable horizontal layout of the box plot;
%PUT =========> PROC BOXPLOT Dataset Variable=&Variable <=========;
PROC BOXPLOT DATA=&Dataset /*(OBS=10)*/;
  PLOT &Variable*Variable / NOTCHES &Orientation BOXSTYLE = SCHEMATICID;
  ID IDvariable /*&Variable &Subject*/;
  INSET NOBS MEAN STDDEV MIN MAX / HEADER='Univariate statistics' POS=TM;
* FORMAT &Variable &Best_Fmt;
RUN;
ODS GRAPHICS OFF;

  %IF (NOT &TrnsType) %THEN %DO;
* - - - - - - - - - - - - - - - ;
* T R A N S F O R M A T I O N S ;
* - - - - - - - - - - - - - - - ;

* Box-Cox plot;
    TITLE2 "Box Cox plot of &Variable from &Data";

*------------------------------------------------------------------;
* Dependent variable must always be positive, if not then add      ;
* ABS(Minimum) + 1 for every record (including the positive ones). ;
* This preceeding transformation does not affect the distribution. ;
* Missing values in the TRANSREG transformation process are always ;
* automatically excluded without generating errors.                ;
*------------------------------------------------------------------;

%PUT =========> Data step Dataset (2) Variable=&Variable <=========;
    DATA &Dataset;
      SET &Dataset;
* Combining Minimum value with every record for addition of ABS(Min);
* (could be done using macro variable from CALL SYMPUT as well) ;
      RETAIN Constant; * for all other records;
      IF (_N_ EQ 1) THEN DO;
        SET &UniStat (KEEP=Minimum); * single record;

* Constant value to add to variable to make all values positive;
        Constant = (-Minimum + 1) * (Minimum LE 0); * lowest value will be 1;
        CALL SYMPUT ("Constant", LEFT(PUT(Constant, 12.6))); * @@@@@@@@@;
      END;

      &Variable = &Variable + Constant; * Retained, calculated Constant;
* no sum statement, because missings must remain missings, not Constant;

* Define (experimental) X variable;
      X = 0; * constant;
      X = RANUNI(0); * between 0 and 1;
    RUN;

    %*PUT --> Constant addition to make &Variable positive is &Constant;

    %IF (&Debug EQ 11) %THEN %DO;
      data sas__out.&Variable; set &Dataset; run;
    %END; %* debug;

* Firstly delete any possible existing BoxCoxP dataset (for recreation);
    %PUT =========> PROC DATASETS / DELETE WORK.BoxCoxP <=========;
    PROC DATASETS LIB=WORK NOLIST; DELETE BoxCoxP; QUIT;
* and perform the Box Cox analysis;
    ODS OUTPUT BoxCoxPlot=BoxCoxP /*Univariate=BoxCoxU ConvergenceStatus=BoxCoxC*/;
*    ODS LISTING;
    ODS GRAPHICS ON;
*    ODS TRACE ON / LABEL;

    %PUT =========> PROC TRANSREG Dataset Variable=&Variable <=========;
* (if missing values left in an error is produced and no BoxCoxPlot created);
    PROC TRANSREG DATA=&Dataset (WHERE=(NOT MISSING(&Variable)));
      MODEL BOXCOX(&Variable / LAMBDA=-5 TO 7 BY .1) = IDENTITY(X);
    %IF (&Debug EQ 12) %THEN %DO;
        OUTPUT OUT=_BOXCOX_;
    %END; %* debug;
    RUN;

    *ODS TRACE OFF;
    ODS GRAPHICS OFF;
    *ODS LISTING CLOSE;
    ODS OUTPUT CLOSE;

    %IF (&Debug EQ 12) %THEN %DO;
      data sas__out.&Variable; set _BoxCox_; run;
    %END; %* debug;
    %IF (&Debug EQ 13) %THEN %DO;
      data sas__out.&Variable._P; set BoxCoxP; run;
/*    data sas__out.&Variable._U; set BoxCoxU; run; * also in results; */
      data sas__out.&Variable._C; set BoxCoxC; run;
    %END; %* debug;

* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - ;
* ODS OUTPUT dataset BoxCoxP contains various Lambda-s          ;
* (within the range specified) and their log-likelyhood values. ;
* The best Lambda is where the LogLike is maximized.            ;
* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - ;

    %IF (%SYSFUNC(EXIST(BoxCoxP))) %THEN %DO;
      %PUT =========> PROC SQL BoxCoxP, Lambda Variable=&Variable <=========;
      PROC SQL NOPRINT;
        SELECT Lambda INTO :Lambda
        FROM BoxCoxP
        HAVING LogLike EQ MAX(LogLike);
      QUIT;

      %LET Lambda = &Lambda; %* lose leading (and possible trailing) spaces;
      %PUT --------- Optimal Lambda values from Box Cox analysis --------- ;
      %PUT Data=&Data, Variable=&Variable, Lambda=&Lambda;

* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - ;
* Prepare second run of normality checking code with transformed data ;
* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - ;

* Calculate classes of transformations from Lambda;
      %IF       (%SYSEVALF(&Lambda LE -1.5)) %THEN %LET Trnsfrmd =  -2;
/*    %ELSE %IF (%SYSEVALF(&Lambda LE -.75)) %THEN %LET Trnsfrmd =  -1;*/
/*    %ELSE %IF (%SYSEVALF(&Lambda LE -.25)) %THEN %LET Trnsfrmd = -.5;*/
      %ELSE %IF (%SYSEVALF(&Lambda LE  -.5)) %THEN %LET Trnsfrmd =  -1; %* chg;
      %ELSE %IF (%SYSEVALF(&Lambda LE  .25)) %THEN %LET Trnsfrmd =   0;
      %ELSE %IF (%SYSEVALF(&Lambda LE  .75)) %THEN %LET Trnsfrmd =  .5;
      %ELSE %IF (%SYSEVALF(&Lambda LE  1.5)) %THEN %LET Trnsfrmd =   1;
      %ELSE                                        %LET Trnsfrmd =   2;

      %IF (%SYSEVALF(&Trnsfrmd EQ 1)) %THEN %DO;
%* Indicate certain informative statistics in &UniStat if no transformation;
        %PUT =========> Data step &UniStat (2) Variable=&Variable <=========;
        DATA &UniStat;
          SET &UniStat; * just one record, every variable/dataset at a time;
          Lambda   = &Lambda;
          Trnsfrmd = &Trnsfrmd;
          IF (zSkewSES GE -2 AND zSkewSES LE 2 AND
              zKurtSEK GE -2 AND zKurtSEK LE 2) THEN Advice = "PARAMETRIC OK";
          ELSE Advice = "PARAMETRIC???";
        RUN; * &UniStat;
      %END;
    %END;

    %ELSE %DO;
      %PUT ######### No Lambda values determined, no BoxCoxP dataset ######### ;
      %PUT Data=&Data, Variable=&Variable, Lambda=UNKNOWN;
    %END;
  %END; %* IF (NOT TrnsType);

%_Append_ (Base=&Stats, Data=&UniStat);

%*put ***DEBUG*** Variable=&Variable TrnsType=&TrnsType Trnsfrmd=&Trnsfrmd Lambda=&Lambda Constant=&Constant;
  %IF (NOT &TrnsType) %THEN %DO;
%* Call normality checks on transformed data;
    %IF (%SYSEVALF(&Trnsfrmd NE 1) AND &ForMyUse NE RECURSIVE) %THEN %DO;
  %* 1 = no transformation;
      %PUT ######### Analyse transformed variables,
                     Data=&Data, Variable=&Variable;
      %DisTrans (Data=&Data, Variable=&Variable, Lambda=&Lambda,
                 Trnsfrmd=&Trnsfrmd, Constant=&Constant, Stats=&Stats,
                 Subject=&Subject, MaxWidth=&MaxWidth, Orientation=&Orientation,
                 ForMyUse=RECURSIVE, Delete=No, Debug=&Debug);
    %END;
  %END; %* IF (NOT TrnsType);
  %ELSE %DO;
* Cleanup: delete temporary datasets;
    %PUT =========> PROC DATASETS / DELETE WORK.&Dataset <=========;
    PROC DATASETS NOLIST; DELETE &Dataset /* and more? */; QUIT;
    %LET SYSLAST = &Stats; %* _LAST_ dataset now points to &Stats;
  %END;

%Skipped:
%Finish:
%MEND DisTrans;
